import UIKit
import Flutter
import AccountKit

@UIApplicationMain
@objc class AppDelegate: FlutterAppDelegate {
    
    var accountKit: AKFAccountKit?
    var methodChanel: FlutterMethodChannel?
    
    override func application(
        _ application: UIApplication,
        didFinishLaunchingWithOptions launchOptions: [UIApplicationLaunchOptionsKey: Any]?
        ) -> Bool {
        
        //init accountKit
        self.accountKit = AKFAccountKit(responseType: AKFResponseType.accessToken);
        
        guard let controller: FlutterViewController = window.rootViewController as? FlutterViewController else {
            return false
        }
        
        self.methodChanel = FlutterMethodChannel.init(name: "authencation_example", binaryMessenger: controller);
        
        methodChanel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            
            switch call.method {
            case "getBatteryLevel":
                self.receiveBatteryLevel(result: result);
            case "signInWithSMS":
                self.signInWithFacebookSMS(result: result);
            case "signInWithEmail":
                self.signInWithFacebookEmail(result: result);
            default:
                result(FlutterMethodNotImplemented);
            }
            
        });
        
        
        
        return super.application(application, didFinishLaunchingWithOptions: launchOptions);
    }
    
    private func receiveBatteryLevel(result: FlutterResult) {
        let device = UIDevice.current;
        device.isBatteryMonitoringEnabled = true;
        if (device.batteryState == UIDeviceBatteryState.unknown) {
            result(FlutterError.init(code: "UNAVAILABLE",
                                     message: "100%",
                                     details: nil));
        } else {
            result(Int(device.batteryLevel * 100));
        }
    }
    
    private func signInWithFacebookSMS(result: FlutterResult) {
        self.showUISignInSMS()
    }
    
    private func signInWithFacebookEmail(result: FlutterResult) {
        self.showUISignInEmail()
    }
    
    
    
}

extension AppDelegate: AKFViewControllerDelegate {
    func prepareLoginViewController(loginViewController: AKFViewController) {
        loginViewController.delegate = self
        //UI Theming - Optional
        loginViewController.uiManager = AKFSkinManager(skinType: .classic, primaryColor: UIColor.blue)
    }
    
    func showUISignInSMS() {
        let inputState = UUID().uuidString
        guard let vc = self.accountKit?.viewControllerForPhoneLogin(with: nil, state: inputState) else {
            return
        }
        vc.enableSendToFacebook = true
        vc.whitelistedCountryCodes = ["VN"];
        self.prepareLoginViewController(loginViewController: vc)
        self.window.rootViewController?.present(vc as UIViewController, animated: true, completion: nil);
    }
    
    func showUISignInEmail() {
        let inputState = UUID().uuidString
        guard let vc = self.accountKit?.viewControllerForEmailLogin(withEmail: nil, state: inputState) else {
            return
        }
        vc.enableSendToFacebook = true
        vc.whitelistedCountryCodes = ["VN"];
        self.prepareLoginViewController(loginViewController: vc)
        self.window.rootViewController?.present(vc as UIViewController, animated: true, completion: nil)
    }
    
    func viewController(_ viewController: (UIViewController & AKFViewController)!, didCompleteLoginWith accessToken: AKFAccessToken!, state: String!) {
        guard let methodChannel = self.methodChanel, let accountKit = self.accountKit else {
            return
        }
        
        accountKit.requestAccount({ (account, error) in
            
            guard let account = account else {
                return
            }
            
            var informations: [String: Any] = [:];
            informations["phone_number"] = account.phoneNumber?.phoneNumber;
            informations["access_token"] = accountKit.currentAccessToken?.tokenString;
            informations["email_address"] = account.emailAddress;
            
            UIAlertView(title: "RESULT", message: account.phoneNumber?.phoneNumber, delegate: nil, cancelButtonTitle: "OK").show()
            
            methodChannel.invokeMethod("didFinishLoggedIn", arguments: informations)
        })
    }
}

